/***************************************************************************
 *   Copyright (C) 2007 by    *
 *      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "abi_gun_vacuum_vlv.h"
#include <qtango.h>
#include <QDesktopWidget>

#include <QKeyEvent>

double zoom_levels[] = {0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 1.1, 1.2, 1.3, 1.4, 1.5, 2};
int zoom_index = 6;

void abi_gun_vacuum_vlv::keyPressEvent(QKeyEvent* e) {
	bool ctrl;
	ctrl = (e->modifiers() & Qt::ControlModifier);
	// cout << "key: " << (ctrl? "ctrl-": "") << e->key() << ", text: " << e->text().toStdString() << endl;
	if (ctrl && e->text()=="=") bigger();
	if (ctrl && e->text()=="-") smaller();
	if (ctrl && e->text()=="0") {zoom_index=5; bigger();}
}

void abi_gun_vacuum_vlv::smaller()
{
	if (zoom_index>0) zoom_index--; else return;
	double resizeFactor = zoom_levels[zoom_index];
	foreach(w_buffer tmp_buffer, ws_buffer) {
		if (tmp_buffer.w==NULL) {
			this->resize(tmp_buffer.width*resizeFactor,tmp_buffer.height*resizeFactor);
		}
		else {
			QFont font = tmp_buffer.font;
			font.setPointSizeF(font.pointSizeF()*resizeFactor);
			tmp_buffer.w->resize(tmp_buffer.width*resizeFactor,tmp_buffer.height*resizeFactor);
			tmp_buffer.w->move(tmp_buffer.x*resizeFactor,tmp_buffer.y*resizeFactor);
			tmp_buffer.w->setFont(font);
		}
	}
}

void abi_gun_vacuum_vlv::bigger()
{
	if (zoom_index<12) zoom_index++; else return;
	double resizeFactor = zoom_levels[zoom_index];
	foreach(w_buffer tmp_buffer, ws_buffer) {
		if (tmp_buffer.w==NULL) {
			this->resize(tmp_buffer.width*resizeFactor,tmp_buffer.height*resizeFactor);
		}
		else {
			QFont font = tmp_buffer.font;
			font.setPointSizeF(font.pointSizeF()*resizeFactor);
			tmp_buffer.w->resize(tmp_buffer.width*resizeFactor,tmp_buffer.height*resizeFactor);
			tmp_buffer.w->move(tmp_buffer.x*resizeFactor,tmp_buffer.y*resizeFactor);
			tmp_buffer.w->setFont(font);
		}
	}
}

abi_gun_vacuum_vlv::abi_gun_vacuum_vlv(QWidget *parent) : QWidget(parent)
{
	QSettings settings("fermi", "acs");
	// load (or set) path for images
	if (settings.contains("imgpath")) {
		imgpath = settings.value("imgpath").toString();
		// cout << imgpath.toStdString() << endl;
	}
	else {
		settings.setValue("imgpath", "/runtime/fermi/share/images/acs");
		// settings.setValue("imgpath", "/home/zambonl/devel/qt/fermi/acs/img");
	}
	if (!imgpath.endsWith("/")) {
		imgpath = imgpath + "/";
	}
	// setup User Interface
	ui.setupUi(this);

	try {
		connect(ui.legendButton, SIGNAL(clicked()), this, SLOT(open_legend()));
		legend_process = new QProcess(this);
	}
	catch (const Tango::DevFailed &e) {
		cout << "cannot connect UI buttons" << endl;
	}
	catch (...) {
		cout << "default exception, cannot setup UI" << endl;
	}	
	line_config(ui.line_linac1_1);
	line_config(ui.line_linac2_1);
	line_config(ui.line_linac2_2);
	line_config(ui.line_linac2_3);
	line_config(ui.line_linac3_1);
	line_config(ui.line_linac3_2);
	line_config(ui.line_linac3_3);
	line_config(ui.line_linac4_1);
	line_config(ui.line_linac4_2);
	line_config(ui.line_linac4_3);
	line_config(ui.line_linac6_1);
	line_config(ui.line_linac6_2);
	line_config(ui.line_linac6_3);
	line_config(ui.line_linac6_4);
	line_config(ui.line_linac6_5);
	line_config(ui.line_linac6_6);
	line_config(ui.line_linac6_7);
	line_config(ui.line_linac7_1);
	line_config(ui.line_undulator3_1);
	line_config(ui.line_undulator3_2);
	line_config(ui.line_undulator3_3);
	line_config(ui.line_undulator3_4);
	line_config(ui.line_undulator3_5);
	line_config(ui.line_undulator3_6);
	line_config(ui.line_undulator3_7);
	line_config(ui.line_undulator4_1);
	line_config(ui.line_undulator4_2);
	line_config(ui.line_undulator4_3);
	line_config(ui.line_undulator4_4);
	line_config(ui.line_undulator4_5);
	line_config(ui.line_undulator4_6);
	line_config(ui.line_undulator4_7);
	line_config(ui.line_padres1_1);
	line_config(ui.line_padres1_2);
	line_config(ui.line_padres1_3);
	line_config(ui.line_padres2_1);
	line_config(ui.line_padres2_2);
	line_config(ui.line_padres2_3);
	line_config(ui.line_padres3_1);
	line_config(ui.line_padres3_2);
	line_config(ui.line_padres3_3);
	line_config(ui.line_padres4_1);

	connect(ui.minus, SIGNAL(clicked()), this, SLOT(smaller()));
	connect(ui.plus, SIGNAL(clicked()), this, SLOT(bigger()));

	w_buffer tmp_buffer;
	// init global container
	tmp_buffer.w = NULL;
	tmp_buffer.x = 0.0;
	tmp_buffer.y = 0.0;
	tmp_buffer.width = (double) this->width();
	tmp_buffer.height = (double) this->height();
	ws_buffer.push_back(tmp_buffer);
	// init all widgets
	foreach(QWidget *w, this->findChildren<QWidget *>()) {
		tmp_buffer.w = w;
		tmp_buffer.font = w->font();
		tmp_buffer.x = (double) w->x();
		tmp_buffer.y = (double) w->y();
		tmp_buffer.width = (double) w->width();
		tmp_buffer.height = (double) w->height();
		ws_buffer.push_back(tmp_buffer);
	}
	QDesktopWidget mydesk;
	cout << "zoom_index: " << zoom_index << ", zoom_levels[zoom_index]: " << zoom_levels[zoom_index] << ", mydesk.width(): " << mydesk.width() << ", mydesk.height(): " << mydesk.height() << endl;
	if (mydesk.width()>100) while (zoom_levels[zoom_index]*this->width() > mydesk.width() && zoom_index>0) zoom_index--;
	if (mydesk.height()>100) while (zoom_levels[zoom_index]*this->height() > mydesk.height()-30 && zoom_index>0) zoom_index--;
	cout << "zoom_index: " << zoom_index << ", zoom_levels[zoom_index]: " << zoom_levels[zoom_index] << ", mydesk.width(): " << mydesk.width() << ", mydesk.height(): " << mydesk.height() << endl;
	if (zoom_index<6) {zoom_index++; smaller();}
}

abi_gun_vacuum_vlv::~abi_gun_vacuum_vlv()
{
	exit(0);
}    


void abi_gun_vacuum_vlv::line_config(TTable *line)
{
	ELabel *childLabel = line->findChild<ELabel *>();
	if(childLabel) {
		childLabel->setFrameShape(QFrame::NoFrame);
		line->setBooleanDisplay(0, "", "", "#DC0000", "#00C800");
	}	
}

void abi_gun_vacuum_vlv::open_legend()
{
	if (legend_process->state() == QProcess::Running) {
		legend_process->kill();
		legend_process->waitForFinished(3000);			
	}	
	legend_process->start("abi_gun_vacuum_vlv_legend");
}
